@@ -12,6 +12,7 @@ __all__ = [
     "check_final_expression",
     "InterpolationParser",
     "parse_statement",
+    "parse_decorator",
     "AssignmentTargetParser",
     "IfElseLoweringParser",
     "BreakContinueConstraint",
@@ -22,6 +23,7 @@ __all__ = [
     "parse_identifier",
     "ImportLocationConstraint",
     "ImportStatementHandler",
+    "parse_python_import",
     "parse_import_name",
     "GlobalNonlocalHandler",
     "parse_name_list",
@@ -73,6 +75,7 @@ from .ast import (
     AstAssignment,
     AstAttribute,
     AstCall,
+    AstDecorator,
     AstDict,
     AstDictItem,
     AstExpression,
@@ -104,12 +107,13 @@ KEYWORD_PATTERN: str = "|".join(rf"\b{kw}\b" for kw in keyword.kwlist)
 TRUE_PATTERN: str = r"\b[tT]rue\b"
 FALSE_PATTERN: str = r"\b[fF]alse\b"
 NULL_PATTERN: str = r"\b(?:null|None)\b"
-IDENTIFIER_PATTERN: str = rf"(?!_mecha_|{KEYWORD_PATTERN})[a-zA-Z_][a-zA-Z0-9_]*\b"
+IDENTIFIER_PATTERN: str = rf"(?!_bolt_|{KEYWORD_PATTERN})[a-zA-Z_][a-zA-Z0-9_]*\b"
+MODULE_PATTERN: str = rf"{IDENTIFIER_PATTERN}(?:\.{IDENTIFIER_PATTERN})*"
 STRING_PATTERN: str = r'"(?:\\.|[^\\\n])*?"' "|" r"'(?:\\.|[^\\\n])*?'"
 NUMBER_PATTERN: str = r"(?:0|[1-9]\d*)(?:\.\d+)?(?:[eE][+-]?\d+)?\b"
 RESOURCE_LOCATION_PATTERN: str = r"(?:\.\./|\./|[0-9a-z_\-\.]+:)[0-9a-z_./-]+"
 
-IMPORT_REGEX = re.compile(rf"^{IDENTIFIER_PATTERN}(?:\.{IDENTIFIER_PATTERN})*$")
+IMPORT_REGEX = re.compile(rf"^{MODULE_PATTERN}$")
 
 
 def get_bolt_parsers(
@@ -163,6 +167,7 @@ def get_bolt_parsers(
             mask=True,
         ),
         "bolt:statement": parse_statement,
+        "bolt:decorator": parse_decorator,
         "bolt:assignment_target": AssignmentTargetParser(
             allow_undefined_identifiers=True,
             allow_multiple=True,
@@ -173,7 +178,12 @@ def get_bolt_parsers(
         "bolt:del_target": parse_del_target,
         "bolt:interpolation": PrimaryParser(delegate("bolt:identifier")),
         "bolt:identifier": parse_identifier,
-        "bolt:import": ImportLocationConstraint(parsers["resource_location_or_tag"]),
+        "bolt:import": AlternativeParser(
+            [
+                ImportLocationConstraint(parsers["resource_location_or_tag"]),
+                parse_python_import,
+            ]
+        ),
         "bolt:import_name": parse_import_name,
         "bolt:global_name": parse_name_list,
         "bolt:nonlocal_name": parse_name_list,
@@ -287,7 +297,20 @@ def get_bolt_parsers(
         "phrase": InterpolationParser("phrase", parsers["phrase"]),
         "greedy": InterpolationParser("greedy", parsers["greedy"]),
         "json": InterpolationParser("json", parsers["json"]),
+        "json_object_entry": InterpolationParser(
+            "json", parsers["json_object_entry"], unpack=r"\*\*"
+        ),
+        "json_array_element": InterpolationParser(
+            "json", parsers["json_array_element"], unpack=r"\*"
+        ),
+        "json_object": InterpolationParser("json_object", parsers["json_object"]),
         "nbt": InterpolationParser("nbt", parsers["nbt"]),
+        "nbt_compound_entry": InterpolationParser(
+            "nbt", parsers["nbt_compound_entry"], unpack=r"\*\*"
+        ),
+        "nbt_list_or_array_element": InterpolationParser(
+            "nbt", parsers["nbt_list_or_array_element"], unpack=r"\*"
+        ),
         "nbt_compound": InterpolationParser("nbt_compound", parsers["nbt_compound"]),
         "nbt_path": InterpolationParser("nbt_path", parsers["nbt_path"]),
         "range": InterpolationParser("range", parsers["range"], fallback=True),
@@ -383,25 +406,27 @@ class UndefinedIdentifier(InvalidSyntax):
 
 def create_bolt_root_parser(parser: Parser):
     """Return parser for the root node for bolt."""
-    return FunctionRootBacktracker(
-        FlushPendingIdentifiersParser(
-            FunctionConstraint(
-                BreakContinueConstraint(
-                    parser=IfElseLoweringParser(parser),
-                    allowed_scopes={
-                        ("while", "condition", "body"),
-                        ("for", "target", "in", "iterable", "body"),
+    return DecoratorResolver(
+        FunctionRootBacktracker(
+            FlushPendingIdentifiersParser(
+                FunctionConstraint(
+                    BreakContinueConstraint(
+                        parser=IfElseLoweringParser(parser),
+                        allowed_scopes={
+                            ("while", "condition", "body"),
+                            ("for", "target", "in", "iterable", "body"),
+                        },
+                    ),
+                    command_identifiers={
+                        "return",
+                        "return:value",
+                        "yield",
+                        "yield:value",
+                        "yield:from:value",
+                        "global:subcommand",
+                        "nonlocal:subcommand",
                     },
-                ),
-                command_identifiers={
-                    "return",
-                    "return:value",
-                    "yield",
-                    "yield:value",
-                    "yield:from:value",
-                    "global:subcommand",
-                    "nonlocal:subcommand",
-                },
+                )
             )
         )
     )
@@ -485,6 +510,7 @@ class InterpolationParser:
     converter: str
     parser: Parser
     prefix: Optional[str] = None
+    unpack: Optional[str] = None
     fallback: bool = False
     final: bool = False
 
@@ -495,11 +521,13 @@ class InterpolationParser:
         for parser, alternative in stream.choose(*order):
             with alternative:
                 if parser == "interpolation":
-                    with stream.syntax(prefix=self.prefix):
+                    with stream.syntax(prefix=self.prefix, unpack=self.unpack):
                         prefix = stream.get("prefix")
+                        unpack = self.unpack is not None and stream.expect("unpack")
                     node = delegate("bolt:interpolation", stream)
                     node = AstInterpolation(
                         prefix=prefix.value if prefix else None,
+                        unpack=unpack.value if unpack else None,
                         converter=self.converter,
                         value=node,
                     )
@@ -521,6 +549,7 @@ def parse_statement(stream: TokenStream) -> Any:
         "bolt:augmented_assignment_target",
         "bolt:assignment_target",
         "bolt:expression",
+        "bolt:decorator",
     ):
         with alternative:
             pending_identifiers.clear()
@@ -585,6 +614,62 @@ def parse_statement(stream: TokenStream) -> Any:
             return node
 
 
+def parse_decorator(stream: TokenStream) -> Any:
+    """Parse decorator."""
+    with stream.syntax(decorator="@"):
+        token = stream.expect("decorator")
+        expression = delegate("bolt:expression", stream)
+        return set_location(AstDecorator(expression=expression), token, expression)
+
+
+@dataclass
+class DecoratorResolver:
+    """Parser for resolving decorators."""
+
+    parser: Parser = required_field()
+
+    def __call__(self, stream: TokenStream) -> AstRoot:
+        node: AstRoot = self.parser(stream)
+
+        stack: List[AstDecorator] = []
+
+        changed = False
+        result: List[AstCommand] = []
+
+        for command in node.commands:
+            if command.identifier == "statement" and isinstance(
+                decorator := command.arguments[0], AstDecorator
+            ):
+                changed = True
+                stack.append(decorator)
+            elif (
+                stack
+                and command.identifier == "def:function:body"
+                and isinstance(command.arguments[0], AstFunctionSignature)
+            ):
+                arg = replace(command.arguments[0], decorators=AstChildren(stack))
+                result.append(
+                    replace(
+                        command, arguments=AstChildren([arg, *command.arguments[1:]])
+                    )
+                )
+                stack.clear()
+            elif stack:
+                exc = InvalidSyntax("Decorators can only be applied to functions.")
+                raise set_location(exc, stack[-1])
+            else:
+                result.append(command)
+
+        if stack:
+            exc = InvalidSyntax("Can not apply decorator to nothing.")
+            raise set_location(exc, stack[-1])
+
+        if changed:
+            node = replace(node, commands=AstChildren(result))
+
+        return node
+
+
 @dataclass
 class AssignmentTargetParser:
     """Parser for assignment targets."""
@@ -902,6 +987,13 @@ class ImportStatementHandler:
         return node
 
 
+def parse_python_import(stream: TokenStream) -> Any:
+    """Parse python import."""
+    with stream.syntax(module=rf"{MODULE_PATTERN}(?=\s)"):
+        token = stream.expect("module")
+        return set_location(AstResourceLocation(path=token.value), token)
+
+
 def parse_import_name(stream: TokenStream) -> AstImportedIdentifier:
     """Parse import name."""
     with stream.syntax(name=IDENTIFIER_PATTERN, comma=r","):
